
#ifndef __LITMUS_PI_SEM_H__
#define __LITMUS_PI_SEM_H__

/* a semaphore maps to a single lso, as with queue locks... if you want to use
 * these as group locks, use an set of resources as  the lso and give the
 * semaphore an appropriate name (e.g., resources A through D in same lso,
 * name lock "ABCD").
 */

/* semaphore handles should not be modified by users */
typedef void*  lso_pi_sem_t;

/* lso_get_pi_sem(handle, name, sem_handle) - obtain a handle for a semaphore
 *
 * A handle is needed for locking and unlocking operations. Returns LSO_OK on
 * success. Stores handle in *sem_handle.
 */
lso_error_t lso_get_pi_sem(lso_t handle, const char* name, 
			   lso_pi_sem_t* sem_handle);

/* lso_pi_semlock(handle) - enter critical section
 */
void lso_pi_semlock(lso_pi_sem_t handle);

/* lso_pi_semunlock(handle) - exit critical section
 */
void lso_pi_semunlock(lso_pi_sem_t handle);

/* lso_release_pi_sem(handle) - release the _handle_ to a pi_sem and sem itself
 */
void lso_release_pi_sem(lso_pi_sem_t *handle);

#endif
