
#ifndef __LITMUS_QLOCK_H__
#define __LITMUS_QLOCK_H__

/* qlock handles should not be modified by users */
typedef void*  lso_qlock_t;

/* lso_create_qlock(handle, name) - create and initialize new qlock
 *
 * This function is guaranteed to behave atomically, i.e., to processes racing
 * to create the same qlock will not intefere in  unpredictable ways. The
 * slower process will always fail in trying to create an already existing lock.
 */
lso_error_t lso_create_qlock(lso_t handle, const char* name);

/* lso_get_qlock(handle, name, lock_handle) - obtain a handle for a qlock
 *
 * A handle is needed for locking and unlocking operations. Returns LSO_OK on
 * success. Stores handle in *lock_handle.
 */
lso_error_t lso_get_qlock(lso_t handle, const char* name, lso_qlock_t* lock_handle);

/* lso_qlock(handle) - enter critical section
 */
void lso_qlock(lso_qlock_t handle);

/* lso_qunlock(handle) - exit critical section
 */
void lso_qunlock(lso_qlock_t handle);

/* lso_release_qlock(handle) - release the _handle_ to a qlock
 * 
 * This does not release the qlock itself. Will set handle to NULL.
 */
void lso_release_qlock(lso_qlock_t *handle);

#endif
