#ifndef __SO_INTERNAL_H__
#define __SO_INTERNAL_H__

#include <stdint.h>

#include "so_internal_arch.h"

#define LSO_MAGIC_1 0x4d54494c  /* LITM */
#define LSO_MAGIC_2 0x4f535355  /* USSO */


struct lso_heap_cell {
	lso_global_t            prev;
	lso_global_t		next;	
	lso_global_t		size;
};

struct lso_heap_info {
	lso_global_t		start;
	lso_global_t		end;
	lso_global_t		free_list_head;
};

struct lso_header {
	/* The file should start with the correct
	 * magic number.
	 */
	uint32_t		magic1;
	uint32_t		magic2;

	/* how many processes can share this object? */
	uint32_t		max_pid;
	
	/* where is the pidmap stored ? */
	lso_global_t		pid_map;
	
	/* where is the dictionary? */
	lso_global_t		dict;

	/* what is the complete size of the file */
	uint32_t		size;

	/* sequence number counter for various purposes*/
	int32_t			seq_no;

	/* how large is the heap? */
	struct lso_heap_info	heap;
};

struct queuelock;

struct litmus_so {
	struct lso_header*	base;
	unsigned int 		pid;
	int			fd;
};

void __lso_lock(struct litmus_so* so);
void __lso_unlock(struct litmus_so* so);

static inline int lso_get_seqno(struct litmus_so* so)
{
	return atomic_inc_return((atomic_t*) &so->base->seq_no);
}

static inline void await_equal(volatile int* var, int value)
{
	while (*var != value)
		cpu_relax();
}

#endif
