
#ifndef __SO_INTERNAL_ARCH_H__
#define __SO_INTERNAL_ARCH_H__

/* collect platform dependent code in here 
 * if we want to port it, it is at least in one place
 */

typedef struct { int counter; } atomic_t;

#define ATOMIC_INIT(i)	{ (i) }

/**
 * atomic_read - read atomic variable
 * @v: pointer of type atomic_t
 * 
 * Atomically reads the value of @v.
 */ 
#define atomic_read(v)		((v)->counter)

/**
 * atomic_set - set atomic variable
 * @v: pointer of type atomic_t
 * @i: required value
 * 
 * Atomically sets the value of @v to @i.
 */ 
#define atomic_set(v,i)		(((v)->counter) = (i))

static __inline__ void atomic_add(int i, atomic_t *v)
{
	__asm__ __volatile__(
		"lock; addl %1,%0"
		:"+m" (v->counter)
		:"ir" (i));
}

/**
 * atomic_add_return - add and return
 * @v: pointer of type atomic_t
 * @i: integer value to add
 *
 * Atomically adds @i to @v and returns @i + @v
 */
static __inline__ int atomic_add_return(int i, atomic_t *v)
{
	int __i;
	__i = i;
	__asm__ __volatile__(
	        "lock; xaddl %0, %1"
		:"+r" (i), "+m" (v->counter)
		: : "memory");
	return i + __i;
}
 
#define atomic_inc_return(v)  (atomic_add_return(1,v))


static __inline__ void cpu_relax(void)
{
	__asm__ __volatile("pause");
}

static __inline__ char test_and_set_char(char x, char* ptr)
{    
	__asm__ __volatile__("xchgb %b0,%1"
			     :"=q" (x)
			     :"m" (*ptr), "0" (x)
			     :"memory");
	return x;
}


#endif

