
#ifndef __SO_SEM_H__
#define __SO_SEM_H__

#include "so_internal_arch.h"
#include "litmus_so.h"
#include "so_qlock.h"
#include "litmus.h"

/*
 * Most of the functions that follow are "wrappers" for system calls. 
 */

static inline void __sem_lock_init(sema_id *sem)
{
	*sem = sema_init();
}

static inline void __semaphore_lock(struct litmus_so* lso, sema_id *sem)
{
 	down(*sem);
}

static inline void __semaphore_unlock(struct litmus_so* lso, sema_id *sem)
{	
	up(*sem);
}

lso_global_t __lso_malloc(struct litmus_so* lso, unsigned int size);
void __lso_free(struct litmus_so* lso, lso_global_t ptr);

static inline lso_global_t __alloc_semaphore(struct litmus_so* lso) 
{
	lso_global_t semlock;
	sema_id* _semlock;

	semlock  = __lso_malloc(lso, sizeof(sema_id));

	if (!semlock)
		return 0;
	
	_semlock    = lso_local_ptr((lso_t) lso, semlock);
	__sem_lock_init(_semlock);

	if (*_semlock < 0)
		return 0;

	return semlock;
}

static inline void __free_semaphore(struct litmus_so* lso, lso_global_t semlock)
{
	sema_id* sem;
	sem = lso_local_ptr((lso_t) lso, semlock);
	sema_free(*sem);
	__lso_free(lso, semlock);	
}

static inline lso_global_t alloc_semaphore(struct litmus_so* lso) 
{
	lso_global_t ret;
	__lso_lock(lso);
	ret = __alloc_semaphore(lso);
	__lso_unlock(lso);
	return ret;
}

static inline void free_semaphore(struct litmus_so* lso, lso_global_t semlock) 
{
	__lso_lock(lso);
	__free_semaphore(lso, semlock);
	__lso_unlock(lso);
}

#endif
