#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "litmus.h"
#include "litmus_so.h"
#include "litmus_pi_sem.h"
#include "so_internal.h"

struct pi_sem {
	struct litmus_so* 	lso;
	int			lock; 
};

static int get_pi_sem_id(struct litmus_so* handle, const char* name)
{
	char buf[32];	
	int lock_no = 0;

	snprintf(buf, 32, "pi-sem/%s", name);
	
	/* check if it exists already */
	while (!lock_no) {
		lock_no = lso_get(handle, buf);
		if (!lock_no) {
			lock_no = lso_get_seqno(handle);
			if (lso_put_exclusive(handle, buf, lock_no) != LSO_OK) 
				lock_no = 0;
		}
	}
	return lock_no;
}


lso_error_t lso_get_pi_sem(lso_t lso, const char* name, 
			   lso_pi_sem_t* sem_handle)
{
	struct litmus_so* handle;
	struct pi_sem* sem;
	int lock_no;
	int od;

	handle = (struct litmus_so*) lso;
	lock_no = get_pi_sem_id(handle, name);

	/*printf("od_open(%d, %d, %d)\n", handle->fd, PI_SEM, lock_no);*/
	od = od_open(handle->fd, PI_SEM, lock_no);
	if (od < 0) {		
		perror("od_open failed");
		return LSO_NO_MEM;
	}

	sem = (struct pi_sem*) malloc(sizeof(struct pi_sem));
	if (!sem)
		return LSO_NO_MEM;
	

	sem->lso  = (struct litmus_so*) handle;
	sem->lock = od;
	*sem_handle = sem;
	return LSO_OK;
}

void lso_pi_semlock(lso_pi_sem_t handle)
{
	struct pi_sem* sem = (struct pi_sem*) handle;
	pi_down(sem->lock);
}


void lso_pi_semunlock(lso_pi_sem_t handle)
{
	struct pi_sem* sem = (struct pi_sem*) handle;
	pi_up(sem->lock);
}


void lso_release_pi_sem(lso_pi_sem_t *handle)
{
	struct pi_sem* sem = (struct pi_sem*) *handle;

	od_close(sem->lock);
	free(sem);
	*handle = NULL;
}

