#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "litmus.h"
#include "litmus_so.h"
#include "litmus_qlock.h"
#include "so_qlock.h"

struct qlock {
	struct litmus_so* 	lso;
	int			slot;
	struct queuelock*	lock;
};

lso_error_t lso_create_qlock(lso_t handle, const char* name)
{
	struct litmus_so* lso = (struct litmus_so*) handle;
	lso_global_t lock;
	char buf[32];	

	snprintf(buf, 32, "qlock/%s", name);

	/* check if it exists already */
	if (lso_get(handle, buf))
		return LSO_EXISTS;

	lock = alloc_queuelock(lso);
	if (lso_put_exclusive(handle, buf, lock) != LSO_OK) {
		free_queuelock(lso, lock);
		return LSO_EXISTS;
	} else
		return LSO_OK;
}


lso_error_t lso_get_qlock(lso_t handle, const char* name, lso_qlock_t* lock_handle)
{
	struct qlock* ql;
	lso_global_t lock;
	char buf[32];	

	snprintf(buf, 32, "qlock/%s", name);
	lock = lso_get(handle, buf);

	if (!lock)
		return LSO_NOT_FOUND;
	ql = (struct qlock*) malloc(sizeof(struct qlock));
	if (!ql)
		return LSO_NO_MEM;
	ql->lso  = (struct litmus_so*) handle;
	ql->lock = lso_local_ptr(handle, lock);
	*lock_handle = ql;
	return LSO_OK;
}

void lso_qlock(lso_qlock_t handle)
{
	struct qlock* ql = (struct qlock*) handle;
	enter_np();
	raw_queue_lock(ql->lso, ql->lock, &ql->slot);
}


void lso_qunlock(lso_qlock_t handle)
{
	struct qlock* ql = (struct qlock*) handle;
	raw_queue_unlock(ql->lso, ql->lock, &ql->slot);
	exit_np();
}


void lso_release_qlock(lso_qlock_t *handle)
{
	struct qlock* ql = (struct qlock*) *handle;
	free(ql);
	*handle = NULL;
}
