#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#include "litmus.h"
#include "litmus_so.h"
#include "litmus_srp_sem.h"
#include "so_internal.h"

struct srp_sem {
	struct litmus_so* 	lso;
	int			lock;
};


static int get_srp_sem_id(struct litmus_so* handle, const char* name)
{
	char buf[32];	
	int lock_no = 0;

	snprintf(buf, 32, "srp-sem/%s", name);
	
	/* check if it exists already */
	while (!lock_no) {
		lock_no = lso_get(handle, buf);
		if (!lock_no) {
			lock_no = lso_get_seqno(handle);
			if (lso_put_exclusive(handle, buf, lock_no) != LSO_OK) 
				lock_no = 0;
		}
	}
	return lock_no;
}

lso_error_t lso_get_srp_sem(lso_t lso, const char* name, 
			    lso_srp_sem_t* sem_handle)
{
	struct litmus_so* handle;
	struct srp_sem* sem;
	int lock_no;
	int od;

	handle = (struct litmus_so*) lso;
	lock_no = get_srp_sem_id(handle, name);

	od = od_open(handle->fd, SRP_SEM, lock_no);
	if (od < 0) {
		perror("od_open failed");	  
		return LSO_NO_MEM;
	}

	sem = (struct srp_sem*) malloc(sizeof(struct srp_sem));
	if (!sem)
		return LSO_NO_MEM;
	

	sem->lso  = (struct litmus_so*) handle;
	sem->lock = od;
	*sem_handle = sem;
	return LSO_OK;
}

void lso_srp_semlock(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	srp_down(sem->lock);
}


void lso_srp_semunlock(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	srp_up(sem->lock);
}


void lso_release_srp_sem(lso_srp_sem_t *handle)
{
	struct srp_sem* sem = (struct srp_sem*) *handle;

	od_close(sem->lock);
	free(sem);
	*handle = NULL;
}


void lso_reg_with_srp_sem(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	reg_task_srp_sem(sem->lock);
}

